<?php
/*
 * Created on 07.07.2008
 *
 * stargazer billing system connectivity support
 *
 */

require_once(dirname(__FILE__).'/constants.php');
if (STARGAZER_CONNECTIVITY_ENABLED) {
	require_once 'Net/Socket.php';
	require_once 'Crypt/Blowfish.php';
	require_once 'XML/Unserializer.php';

	PEAR::setErrorHandling(PEAR_ERROR_RETURN);
	define('CRYPT_BLOWFISH_NOMCRYPT', true);
}

class STARGAZER_CONNECTION {
	public $conn_ok;
	private $socket;
	private $cryptor;

	function STARGAZER_CONNECTION() {
		$this->socket = new Net_Socket();

		//$this->cryptor = new Crypt_Blowfish(str_pad(STARGAZER_ADMIN_PWD, STARGAZER_ADMIN_PWD_LEN, chr(0), STR_PAD_RIGHT));
		$this->cryptor =& Crypt_Blowfish::factory('ecb', str_pad(STARGAZER_ADMIN_PWD, STARGAZER_ADMIN_PWD_LEN, chr(0), STR_PAD_RIGHT));
		if (PEAR::isError($this->cryptor)) {
			$this->logmsg("Error initializing encryption code: ".$this->cryptor->getMessage());
			return;
		}
//		$this->cryptor->setKey(str_pad(STARGAZER_ADMIN_PWD, STARGAZER_ADMIN_PWD_LEN, chr(0), STR_PAD_RIGHT));

//		$this->sg_cash_add('qwer', '1', 'q');

//		if ($this->sg_connect()) {
//			if (!$this->sg_process_header() || !$this->sg_process_login() || !$this->sg_process_pwd()) {
//				$this->sg_disconnect();
//				return;
//			}
//			/*<SetUser><login value="qwer"/><cash add="1.000000" msg=""/></SetUser>*/
//			//$this->sg_process_data("<SetUser><login value=\"qwer\"/><cash add=\"1.000000\" msg=\"\"/></SetUser>");
//			$this->sg_process_data("<GetUser login=\"qwer\"/>");
//			$this->sg_disconnect();
//		}
	}

	/**
	 * Make connection to stargazer
	 */
	function sg_connect() {
		$this->socket->connect(STARGAZER_HOST, STARGAZER_PORT, false, 30);

		if (PEAR::isError($this->socket)) {
			$this->conn_ok = false;
			$this->logmsg("error connecting to ".STARGAZER_HOST.":".STARGAZER_PORT."!\n".print_r($this->socket, true));
			return false;
		}
		$this->logmsg("connected to ".STARGAZER_HOST.":".STARGAZER_PORT);
		return true;
	}

	/**
	 * Disconnect from stargazer
	 */
	function sg_disconnect() {
		if ($this->socket->disconnect()) {
			$this->logmsg("disconnected from ".STARGAZER_HOST.":".STARGAZER_PORT);
		}
		return true;
	}

	/**
	 * encrypt plaintext using blowfish
	 */
	function encryptdata($data) {
		return $this->cryptor->encrypt($data);
	}

	/**
	 * decrypt data using blowfish
	 */
	function decryptdata($data) {
		return $this->cryptor->decrypt($data);
	}

	/**
	 * Internal message logging
	 */
	function logmsg($msg) {
		global $logger;

		$logger->log("CONN_STARGAZER: ".$msg);
		return true;
	}

	/**
	 * encode12 util
	 * e.g.: afchpgihjh
	 */
	function encode12($insrc, $len) {
		$src = str_pad($insrc, $len, chr(0), STR_PAD_RIGHT);
		$src[$len] = chr(0);
		$dst = str_repeat(chr(0), $len * 2 + 1);

		for ($i = 0; ; $i++) {
			if ($src[$i] == chr(0))
				break;
			$c1 = ord($src[$i]) & 0x0F;
			$c2 = (ord($src[$i]) & 0xF0) >> 4;

			$c1 += ord('a');
			$c2 += ord('a');

			$dst[$i*2] = chr($c1);
			$dst[$i*2 + 1] = chr($c2);
		}
		$dst[$i*2] = chr(0);

		return trim($dst);
	}

	/**
	 * decode21 util
	 * e.g.: afchpgihjh
	 */
	function decode21($src) {
		$len = strlen($src);
		$dst = str_repeat(chr(0), $len / 2);

		for ($i = 0; $i < $len / 2; $i++) {
			$c1 = ord($src[$i*2]);
			$c2 = ord($src[$i*2 + 1]);

			$c1 -= ord('a');
			$c2 -= ord('a');

			$dst[$i] = chr($c1 + ($c2 << 4));
		}

		return $dst;
	}

	/**
	 * send/recv header
	 */
	function sg_process_header() {
		$this->socket->write(STARGAZER_STG_HEADER);
		$res = $this->socket->read(strlen(STARGAZER_STG_HEADER));
		if ($res == STARGAZER_OK_HEADER) {
			$this->logmsg("header exchange: header ok");
			return true;
		} elseif ($res == STARGAZER_ERR_HEADER) {
			$this->logmsg("header exchange: header incorrect");
		} else {
			$this->logmsg("header exchange: unknown error");
		}
		return false;
	}

	/**
	 * send/recv username
	 */
	function sg_process_login() {
		$str = str_pad(STARGAZER_ADMIN_LOGIN, STARGAZER_ADMIN_LOGIN_LEN, chr(0), STR_PAD_RIGHT);
		$this->socket->write($str);
		$res = $this->socket->read(strlen(STARGAZER_OK_LOGIN));
		if ($res == STARGAZER_OK_LOGIN) {
			$this->logmsg("login exchange: login ok");
			return true;
		} elseif ($res == STARGAZER_ERR_LOGIN) {
			$this->logmsg("login exchange: login incorrect");
		} else {
			$this->logmsg("login exchange: unknown error");
		}
		return false;
	}

	/**
	 * send/recv password
	 */
	function sg_process_pwd() {
		$str = str_pad(STARGAZER_ADMIN_LOGIN, STARGAZER_ADMIN_LOGIN_LEN, chr(0), STR_PAD_RIGHT);
		$this->socket->write($this->encryptdata($str));
		$res = $this->socket->read(strlen(STARGAZER_OK_LOGINS));
		if ($res == STARGAZER_OK_LOGINS) {
			$this->logmsg("password exchange: password ok");
			return true;
		} elseif ($res == STARGAZER_ERR_LOGINS) {
			$this->logmsg("password exchange: password incorrect");
		} else {
			$this->logmsg("password exchange: unknown error");
		}
		return false;
	}

	/**
	 * send/recv misc data
	 */
	function sg_process_data($data) {
//		$this->logmsg("data exchange: $data");

		$len = strlen($data);
		$str = $data;
		if (strlen($str) % 8 == 0)
			$str .= chr(0);
		while (strlen($str) % 8 != 0)
			$str .= chr(0);
		//$str = str_pad($data, $len + (($len + 1) % 8), chr(0), STR_PAD_RIGHT);
		$this->socket->write($this->encryptdata($str));

		//read answer until chr(0)
		$resxml = '';
		while (true) {
			$temp = $this->socket->read(8);

			if (PEAR::isError($temp)) {
				$this->logmsg("error reading socket: ".print_r($temp, true));
				return false;
			}
			if ($temp == NULL)
				break;

			$temp = $this->decryptdata($temp);
			$zpos = strpos($temp, chr(0));
			if ($zpos === false) {
				$resxml .= $temp;
			} else {
				$resxml .= substr($temp, 0, $zpos);
				break;
			}
		}
		$this->logmsg("data exchange: $data => $resxml");
		$res = $this->xml2array($resxml);
		if ($res !== false)
			return $res;
		return false;
	}

	/**
	 * xml2array
	 */
	function xml2array($xml) {
		$options = array(
						XML_UNSERIALIZER_OPTION_COMPLEXTYPE => 'array',
//						XML_UNSERIALIZER_OPTION_ATTRIBUTE_KEY => 'login',
						XML_UNSERIALIZER_OPTION_ATTRIBUTES_PARSE => true
						);
		$unserializer = new XML_Unserializer($options);
		if ($unserializer->unserialize($xml) === true) {
			$data = $unserializer->getUnserializedData();
			$this->logmsg("xml2array:\n".print_r($data, true));
			return $data;
		}
		return false;
	}

	/**
	 * add to cash
	 */
	function sg_cash_add($user, $summ, $addinfo = null) {
		$this->logmsg("sg_cash_add('$user', '$summ', '$addinfo')");
		if ($this->sg_connect()) {
			if (!$this->sg_process_header() || !$this->sg_process_login() || !$this->sg_process_pwd()) {
				$this->sg_disconnect();
				return false;
			}

//			<SetUser><login value="qwer"/><cash add="1.000000" msg=""/></SetUser>
			if ($addinfo == null)
				$addinfo = '';
			else
				$addinfo = $this->encode12(convert_cyr_string(STARGAZER_OPERATION_STKCASHTRANSFER_INFO." ($addinfo)", 'w', 'k'), STARGAZER_LOG_MSG_LEN);
			$res = $this->sg_process_data("<SetUser><login value=\"$user\"/>" .
				"<cash add=\"$summ\" msg=\"$addinfo\"/>" .
				"</SetUser>");

			//parse reply. e.g. <SetUser result="ok"/>
			if (is_array($res) && ($res['result'] == 'ok')) {
				$res = true;
			} else
				$res = false;

			$reslog = $res == true ? 'ok' : 'error';
			$this->logmsg("sg_cash_add = $reslog");

			$this->sg_disconnect();

			return $res;
		}
		return false;
	}

	/**
	 * get user info
	 */
	function sg_getuser($user) {
		$this->logmsg("sg_getuser('$user')");
		if ($this->sg_connect()) {
			if (!$this->sg_process_header() || !$this->sg_process_login() || !$this->sg_process_pwd()) {
				$this->sg_disconnect();
				return false;
			}

//			<GetUser login="qwer" />
			$res = $this->sg_process_data("<GetUser login=\"$user\" />");

			//parse reply. e.g. <SetUser result="ok"/>
			if (!is_array($res) && ($res['result'] == 'ok'))
				$res = false;

			$reslog = $res == true ? 'ok' : 'error';
			$this->logmsg("sg_getuser = $reslog");

			$this->sg_disconnect();

			return $res;
		}
		return false;
	}

}; //class STARGAZER_CONNECTION

$conn_stargazer = STARGAZER_CONNECTIVITY_ENABLED ? new STARGAZER_CONNECTION : null;

?>
